function [sol_RK4,avg_num_iter] = local_patch_tracker_rk4

% Purpose: to output all solutions for the fundamental matrix, E in
% projective space.  This is done by tracking all paths from start system
% to target system.

% Gather data used to set up homotopy
[Q, N_star, N_hat] = local_patch_setup;

% Set tracking tolerance
eps = 10^(-6);

% Read in start values for ODE solver from Bertini file (pre-processing)
% Here, there are 10 solution paths.
start_values = fopen('start_10','r');
fscanf(start_values,'%d',1);
p_star_10_sol = fscanf(start_values,'%e');
fclose(start_values);
start_10_values = vec2mat(p_star_10_sol,2);
start_10_complex = start_10_values(:,1) + start_10_values(:,2)*1i;
start_10 = vec2mat(start_10_complex,9);

sol_RK4 = zeros(9,10);
total_num_iter = 0;

% Loop to get all 10 solutions from path data using parameter homotopy continuation
for i = 1:10
    
    % Set initial step size
    dt = 0.1;

    % Track from t = 1 to t = 0
    t = 1;

    % Set start data for specific path
    RK4 = pinv(N_star)*transpose(start_10(i,:));

    % Initialize counter for convergence tracking and ODE comparison
    consec_convergence_RK4 = 0;
    num_loops = 0;
    iter_counter_RK4 = 0;

    % Start tracking!
    while t > 0 && num_loops < 1000

        % Move from projective space to affine space using coordinate-wise patching
        [~,c] = max(abs(RK4));
        v = RK4/RK4(c);

        RK4 = v([1:c-1,c+1:end]);

        tnew = max(t - dt,0);
        if tnew == 0
            dt = t;
        end

        num_loops = num_loops + 1;

        % Use ODE solver to predict to tnew
        % Use RK4 and max 3 Newton steps
        
        % Evaluate homotopy function
        [~, dGy, dGt] = local_patch_evalhom(RK4,t,Q,N_star,N_hat,c);
        f = dGy\dGt*dt;

        % RK4 --> classical Runge-Kutta method (4th order)
        F1rk4 = f;

        [~, dGy, dGt] = local_patch_evalhom(RK4 + 0.5*F1rk4,tnew+0.5*dt,Q,N_star,N_hat,c);
        f = dGy\dGt*dt;

        F2rk4 = f;

        [~, dGy, dGt] = local_patch_evalhom(RK4 + 0.5*F2rk4,tnew+0.5*dt,Q,N_star,N_hat,c);
        f = dGy\dGt*dt;

        F3rk4 = f;

        [~, dGy, dGt] = local_patch_evalhom(RK4 + F3rk4,tnew,Q,N_star,N_hat,c);
        f = dGy\dGt*dt;

        F4rk4 = f;

        x1 = RK4 + (1/6)*(F1rk4+2*F2rk4+2*F3rk4+F4rk4);

        % Use max 3 Newton steps to try to correct back to path
        % Starting values are from ODE output
        max_newton = 4;
        for j = 1:max_newton
            [G, dGy, ~] = local_patch_evalhom(x1,tnew,Q,N_star,N_hat,c);
            % Test for convergence for RK4
            % If converges, make another time step
            % If have 3 consecutive convergences, make time step 2 times larger
            % If have failed convergence, shorten time step by 1/2 and try
            % again
            if norm(G,2) < eps
                t = tnew;
                RK4 = x1;
                consec_convergence_RK4 = consec_convergence_RK4 + 1;
                iter_counter_RK4 = iter_counter_RK4 + 1;
                if consec_convergence_RK4 >= 3
                    dt = 2*dt;
                    consec_convergence_RK4 = 0;
                end
                break
            else
                if j < max_newton
                    x1 = x1 - dGy\G;
                else
                    dt = 0.5*dt;
                    [G, dGy, ~] = local_patch_evalhom(RK4,t,Q,N_star,N_hat,c);
                    RK4 = RK4 - dGy\G;
                    consec_convergence_RK4 = 0;
                end
            end
        end

        % Add coordinate rescaled to 1 for patch back into data vector
        RK4 = [RK4(1:c-1);1;RK4(c:end)];

    end

    % Save data to solution vector
    sol_RK4(:,i) = N_hat*RK4;
    
    % Add to total number of iterations for all solution paths
    total_num_iter = total_num_iter + iter_counter_RK4;

end

% Calculate average number of iterations over 10 solution paths
avg_num_iter = total_num_iter/10;

save MATfilename

return