function [sol_RK4,avg_num_iter] = early_trunc_tracker_rk4_6pt

[p_hat, p_star] = early_trunc_setup_6pt;

% tracking tolerance
eps = 10^(-7);

% Final tolerance
final_tol = 10^(-3);

% starting values for ODE are from output of Bertini known system
% in the file soln_52 have start values for ODE solver
% first number is real part and second is imaginary part
% there are 52 data points of 10 coordinates - want to map each sol. path
start_values = fopen('soln_52','r');
fscanf(start_values,'%d',1);
p_star_52_sol = fscanf(start_values,'%e');
fclose(start_values);
start_52_values = vec2mat(p_star_52_sol,2);
start_52_complex = start_52_values(:,1) + start_52_values(:,2)*1i;
start_52 = vec2mat(start_52_complex,10);

sol_RK4 = zeros(10,52);
total_num_iter = 0;

% Create data arrays for early truncation
t_data = zeros(52,50);
t_data(:,1) = 1;
E_data = zeros(10,52,50);
slope_data = zeros(1,52,50);
slope0_data = zeros(1,52,50);
sol_real = zeros(52,1);
break0 = zeros(52,1);
graph_data = zeros(52,50);

% loop to get all 10 start points for the solution paths
for i = 1:52
    
    % initial step size
    dt = 0.1;

    % track from t = 1 to t = 0
    t = 1;
    
    RK4 = transpose(start_52(i,:));
    E_data(:,i,1) = transpose(start_52(i,:));
    graph_data(i,1) = norm(transpose(start_52(i,:)),2);

    % initialize counter for convergence tracking and ODE comparison
    consec_convergence_RK4 = 0;
    num_loops = 0;
    iter_counter_RK4 = 0;

    while t > 0 && num_loops < 1000
        
        [~,c] = max(abs(RK4(1:9,1)));
        RK4(1:9,1) = RK4(1:9,1)/RK4(c,1);
        RK4 = RK4([1:c-1,c+1:end],1);

        tnew = max(t - dt,0);
        if tnew == 0
            dt = t;
        end

        num_loops = num_loops + 1;
        
        % choose the randomizing matrix
        [Rand] = early_trunc_rand_6pt(RK4,c);

        % use ODE solver to predict to tnew
        % evaluate homotopy function
        [~, dGy, dGt] = early_trunc_evalhom_6pt(RK4,t,p_star,p_hat,c,Rand);
        f = dGy\dGt*dt;

        % RK4 --> classical Runge-Kutta method (4th order)
        F1rk4 = f;

        [~, dGy, dGt] = early_trunc_evalhom_6pt(RK4 + 0.5*F1rk4,tnew+0.5*dt,p_star,p_hat,c,Rand);
        f = dGy\dGt*dt;

        F2rk4 = f;

        [~, dGy, dGt] = early_trunc_evalhom_6pt(RK4 + 0.5*F2rk4,tnew+0.5*dt,p_star,p_hat,c,Rand);
        f = dGy\dGt*dt;

        F3rk4 = f;

        [~, dGy, dGt] = early_trunc_evalhom_6pt(RK4 + F3rk4,tnew,p_star,p_hat,c,Rand);
        f = dGy\dGt*dt;

        F4rk4 = f;

        x1 = RK4 + (1/6)*(F1rk4+2*F2rk4+2*F3rk4+F4rk4);

        % use max 3 Newton steps to try to correct
        % starting values are from ODE output
        max_newton = 4;
        success_newton = 0;
        for j = 1:max_newton
            [G, dGy, ~] = early_trunc_evalhom_6pt(x1,tnew,p_star,p_hat,c,Rand);
            % test for convergence for RK4
            if norm(G,2) < eps
                success_newton = 1;
                t = tnew;
                RK4 = x1;
                consec_convergence_RK4 = consec_convergence_RK4 + 1;
                iter_counter_RK4 = iter_counter_RK4 + 1;
                if consec_convergence_RK4 >= 3
                    dt = 2*dt;
                    consec_convergence_RK4 = 0;
                end
                break
            else
                if j < max_newton
                    x1 = x1 - dGy\G;
                else
                    dt = 0.5*dt;
                    [G, dGy, ~] = early_trunc_evalhom_6pt(RK4,t,p_star,p_hat,c,Rand);
                    RK4 = RK4 - dGy\G;
                    consec_convergence_RK4 = 0;
                end
            end
        end
        
        RK4 = [RK4(1:c-1,1); 1; RK4(c:end,1)];
        
        if success_newton

            % Update solution vector (9x1)
            sol_RK4(:,i) = RK4;
            
            % Add E, t data to array for use in early truncation
            E_data(:,i,iter_counter_RK4+1) = sol_RK4(:,i);
            t_data(i,iter_counter_RK4+1) = t;
            graph_data(i,iter_counter_RK4+1) = norm(imag(sol_RK4(:,i)),2);

            % Value stored if at end of tracking
            % Test whether imag part of sol < some tolerance -- real?
            sol_real(i,1) = norm(imag(sol_RK4(:,i)),inf) < 10*final_tol;

            % Add early truncation step
            % Start at t < 0.3 since at that point paths include influence from
            % target system

             if final_tol < t && t < 0.3 && iter_counter_RK4 >= 2
                
                % Find 2-norms of imaginary part of E data
                e1 = norm(imag(E_data(:,i,iter_counter_RK4+1)),2);
                e2 = norm(imag(E_data(:,i,iter_counter_RK4)),2);
                % Calculate slope from "path" formed from e1, e2, etc.
                % This would lead us to an "endpoint"
                slope_data(1,i,iter_counter_RK4) = (e1-e2)/(t_data(i,iter_counter_RK4+1)-t_data(i,iter_counter_RK4));   
                % Calculate slope from e1 to origin (0,0)
                slope0_data(1,i,iter_counter_RK4) = e1/t_data(i,iter_counter_RK4+1);
 
                % Find "endpoint" (ep) based on slope_data and e1 along y-axis (t=0)
                ep = e1 - (slope_data(1,i,iter_counter_RK4)*t_data(i,iter_counter_RK4+1));
                % Find distance from e1 to "endpoint" (d1), e1 to origin (d2),
                % and "endpoint" to origin (d3)
                % These distances are squared for ease of calculations
                d1 = (e1 - ep)^2 + (t_data(i,iter_counter_RK4+1))^2;
                d2 = e1^2 + (t_data(i,iter_counter_RK4+1))^2;
                d3 = ep^2;
                
                % Use law of cosines to find angle between line mapping e1 to
                % "endpoint" and line mapping e1 to the origin
                % This angle is labeled a3 (opposite d3 side)
                a3 = acos((d2 + d1 - d3)/(2*sqrt(d1)*sqrt(d2)));
 
                % Want to break loop if angle is "too large"
                % Here, "too large" is an angle above 30 degrees
                if a3 > (5*pi/6)
                    sol_real(i,1) = 0; % Endpoint is non-real
                    break0(i,1) = 1; % Marks that a break occurred 
                    break
                end
                
             end

        end

    end

    % Save data to solution vector
    sol_RK4(:,i) = RK4;
    
    % Add to total number of iterations for all solution paths
    total_num_iter = total_num_iter + iter_counter_RK4;

end

% for k = 1:52
%     plot(t_data(k,:),graph_data(k,:))
%     hold on
% end

% Calculate average number of iterations over 52 solution paths
avg_num_iter = total_num_iter/52;

save MATfilename

return